/**
*	File: mst_stepper.js
*	Version: 1.0
*	Author: Wellcom Worldwide
*
*	Description: 
*
*	This file contains the stepper functionality used for Mercer Style Guide
*	Allows for vertical and horizontal initializations.
*	
*/

;(function($) {



	// Check that jQuery exists.
	if(typeof $ == "undefined" && typeof jQuery == "undefined") {
		throw new Error('JQuery must be loaded.');
	} else if ( typeof $ == "undefined" && typeof jQuery != "undefined") {
		var $ = jQuery;
	}



	$.fn.__mstTabs = function ( opts ) {


		var selectedObjects = this;


		var settings = $.extend({
			mobileBreakpoint: 768,
			dropdownBreakpoint: 500,
			tabHeaderClass: ".__mst_tab_header",
			tabItemClass: ".__mst_tab_list_item",
			tabListClass: ".__mst_tab_list",
			equalWidth: false,
			handleOverflow: true,
			responsizeTabs: true,
			moreButtonContent: '<span class="__mst_tabs_dropdown_title">More</span> <i class="mericon-angle-down"></i>'
		}, opts);


		var visibilityDefaults = {
	        callback: function () { },
	        runOnLoad: true,
	        frequency: 100,
	        previousVisibility : null
	    };


	    settings.activeItem = null;


		// protected objects
		var calculations = {
			width: $(window).outerWidth(),
			height: $(window).outerHeight()
		};

		// Store all of the original list items here for usage later.
		var originalList = null;


		// Dynamic List - After-Sorting
		var showingList = [];
		var dropdownList = [];


		var pluginMethods = {
			__init: function ( selectedObjects ) {
				// console.log('initialized');

				$.each(selectedObjects, function( i , o ) {

					var selector = $(this);

					// Ensure that the equalizer class is set.
					if( !selector.hasClass('__mst_tabs') ) {
						selector.addClass('__mst_tabs');
					}

					// Also run a visibility check on each of the selectors
					if(!selector.is(":visible")) {
						// console.log(selector);
						// console.log('Invisible item has been detected.');
						pluginMethods.checkVisibility( selector, $.extend(
							visibilityDefaults,
							{
								callback: function() {

									// console.log(' Visibility has changed!');

									pluginMethods.tabify( selector );
								}
							}
							) );
					} else {
						
						pluginMethods.tabify( selector );
					}

					$(window).on('resize', function(ev) {
						// console.log('resize detected!');
						pluginMethods.tabify( selector );
					});

				});

			},
			rerenderList: function ( tabList, tabItems ) {

				// This method is used to remove the white-space that is rendered by the 
				// browser on inline-block elements
				tabList.empty();

				$.each(tabItems, function( i , o) {
					item = $(this);

					if( !item.is( settings.activeItem ) ) {
						item.removeClass('active');
					}

					tabList.append(item);

					var domItem = tabList.find( item );

					// Now to apply the event listener onto the appended item
					domItem.off('click tap', pluginMethods.onClickTab)
						.on('click tap', { 
							tabList: tabList, 
							showingList: showingList, 
							dropdownList: dropdownList }, 
							pluginMethods.onClickTab
						);


					// item.remove();
				});


			},
			tabify: function( selector ) {


				// Search for the tab header field
				var headerWrapper = selector.find( settings.tabHeaderClass );
				var tabList = headerWrapper.find( settings.tabListClass );
				var tabItems = tabList.find( settings.tabItemClass );

				// Run on the first time
				if( originalList === null) {
					originalList = tabItems;
					settings.activeItem = originalList[0];
				}





				// @Inject: Re-render the list to remove character spacing.
				pluginMethods.rerenderList( tabList, tabItems );

				var tabListWidth = tabList.outerWidth();
				var selectorWidth = selector.outerWidth();

			
				var lists = pluginMethods.sortListItems( originalList, tabList );

				var showingList = lists.showingList;
				var dropdownList = lists.dropdownList;


				if( dropdownList.length > 0 ) {

					// An item has exceeded the maximum tab width size; consider using 
					// the extra dropdown menu.

					if( settings.responsizeTabs ) {

						// Continue on, responsify the tabs into dropdown menu.
						var buttonContent = settings.moreButtonContent;

						// Hide all of the current tabs and then create a virtual element
						// for usage. 


						var moreButton = tabList.find('li.mst-tabs-more');
						
						if(moreButton.length > 0) {
							moreButton.remove();
						}

						if( showingList.length == 0 && dropdownList.length == originalList.length && $(window).width() <= settings.mobileBreakpoint ) {
						
							var moreButton = "<li class='__mst_tabs_more dropdown'>";

						} else {

							// CHeck to see if a div exists for dropdowns within the list element
							var moreButton = "<li class='__mst_tabs_more'>";

						}

								moreButton += "<div>";


									moreButton += "<span class='__mst_tabs_dropdown_selector'>" + buttonContent + "</span>";
									// moreButton += "<span style='font-weight: bold; display: block; width: 100%; text-align: center;'>more</span>";
								moreButton += "</div>";


								// Actual List

								moreButton += "<div class='__mst_tabs_dropdown'>";

									moreButton += pluginMethods.renderDropdownList( dropdownList );

								moreButton += "</div>";


							moreButton += "</li>";

						// Now Rerender the list before appending
						pluginMethods.rerenderList( tabList, showingList );


						tabList.append( moreButton );





						// Once appended, bind the onclick listener to the new more button.
						var moreButtonDomElement = tabList.find('li.__mst_tabs_more');

						if( moreButtonDomElement.length > 0 ) {


							// Also add an event listener for when the user clicks / taps outside
							// of the designated zone for clicking, this should trigger a disable/hide
							// of the dropdown menu within the more button
							$(document).off('click tap', pluginMethods.offClickDocument)
								.on('click tap', { tabList: tabList }, pluginMethods.offClickDocument);



							// Dom element has been found, now to bind the event listener. 
							moreButtonDomElement.off('click tap', pluginMethods.onClickMore)
								.on('click tap', { tabList: tabList },  pluginMethods.onClickMore);



							// Now to bind an event to each of the list items within the dropdown menu


							var dropdownListItems = moreButtonDomElement.find('div.__mst_tabs_dropdown_item');


							if( dropdownListItems.length > 0 ) {




								// Begin the Binding Agent process
								if( showingList.length == 0 && dropdownList.length == originalList.length && $(window).outerWidth() <= settings.mobileBreakpoint ) {
									var hasDropdownTitleSet = null;
								}

								
								$.each( dropdownListItems, function( i , o ) {

									var ddItem = $(this);


									if(ddItem.hasClass('active')) {

										// Make More Button active
										moreButtonDomElement.addClass('active');

									}


									ddItem.off('click tap', pluginMethods.onClickMoreItem)
										.on('click tap', { tabList: tabList, showingList: showingList, dropdownList: dropdownList }, pluginMethods.onClickMoreItem);


									// Also run through and deactivate 


									if( showingList.length == 0 && dropdownList.length == originalList.length && $(window).outerWidth() <= settings.mobileBreakpoint ) {

										if ( ddItem.attr('data-target') == $(settings.activeItem).attr('data-target') ) {

											ddItem.addClass('active');
											var tabTitle = ddItem.html();
											var dropdownTitle = moreButtonDomElement.find('.__mst_tabs_dropdown_title');
											dropdownTitle.html( tabTitle );
											hasDropdownTitleSet = true;

										}

									}



								});


								if( showingList.length == 0 && dropdownList.length == originalList.length && $(window).outerWidth() <= settings.mobileBreakpoint ) {
									if( hasDropdownTitleSet === null ) {

										// console.log($( dropdownList[0] ).html());

										// Apply the first tab title
										var firstTab = $( dropdownListItems[0] );
										firstTab.addClass('active');
										var tabTitle = firstTab.html();
										var dropdownTitle = moreButtonDomElement.find('.__mst_tabs_dropdown_title');
										dropdownTitle.html( tabTitle );

										// Now to find the correspoinding dropdown item and apply the active class onto it


									}
								}


							}




						}


					}

				} else {

					pluginMethods.rerenderList( tabList, showingList );


				}


			},
			sortListItems: function( originalList, tabList ) {

				// Retrieve the TabList Width and begin the caluclations based off of the original list
				// that is passed from the calling constructor method.
				var tabListWidth = tabList.innerWidth();
				var currentTabListWidth = 0;
				var tabItemBreakpoint = null;



				// Before flushing the array, see how many are inside the dropdown list


				// Reset the other lists
				showingList = [];
				dropdownList = [];


				if( $(window).width() > settings.mobileBreakpoint ) {

					// Append to a fake DOM Element that is used to measure the
					// element's real size, required for outerWidth!

					if( $('body').find('.__mst_tabs_faker').length <= 0 ) {
						$('body').append('<div class="__mst_tabs_faker __mst_tabs"><div class="__mst_tab_header"><ul class="__mst_tab_list"></ul></div></div>');
					}




					var fakerElement = $('body').find('.__mst_tabs_faker');
					var fakerUl = fakerElement.find('ul.__mst_tab_list');
					fakerUl.css('width', tabListWidth + 'px');

					/**
					*	@INJECT: Run a faker on the more button itself to determine the outerwidth
					*	of the item so that a true calculation can be determined for whether or not
					*	two items need to be removed from the dom stack
					*/

					// Include the elippsis
					// var buttonContent = '&hellip;';
					var buttonContent = settings.moreButtonContent;

					// CHeck to see if a div exists for dropdowns within the list element
					var moreButton = "<li class='__mst_tabs_more'>";

							moreButton += "<div>";
								moreButton += "<span class='__mst_tabs_dropdown_selector'>" + buttonContent + "</span>";
							moreButton += "</div>";


							// Actual List

							moreButton += "<div class='__mst_tabs_dropdown'>";

							moreButton += "</div>";


						moreButton += "</li>";



					// Append it into the faker element 
					fakerUl.append( moreButton );

					var moreButtonDom = fakerUl.find('li.__mst_tabs_more');


					// Retrieve the outer width
					var moreButtonWidth = moreButtonDom.outerWidth( true );
					// console.log( moreButtonWidth );

					// console.log(moreButtonWidth);


					// Now remove it from the dom stack.
					moreButtonDom.remove();

					/**
					*	@END INJECT
					*/

					// console.log('New list');
					// console.log('UL Width:' + tabListWidth);


					$.each( originalList, function( i , o ) {

						// Declarations of the item
						var item = $(this);

						fakerUl.append( item );

						var fakerItem = fakerUl.find( item );

						// console.log( fakerItem );

						var itemWidth = fakerItem.outerWidth( true );



						// console.log('   ');
						// console.log(' Tab width:  ' + tabListWidth);
						// console.log(' Current width increment:  ' + currentTabListWidth);
						// console.log(' Estimated new width: ' + parseInt(currentTabListWidth + itemWidth) );

						if( tabListWidth <= parseInt(currentTabListWidth + itemWidth + moreButtonWidth) ) {

							// console.log( itemWidth );
							// Place this item into the dropdown list
							dropdownList.push( item );

						} else {

							showingList.push( item );

						}

						// Append the width to the main outer count 
						currentTabListWidth += itemWidth;

						// Also check to see if the tabItem breaking point has already been breached.
						if( tabItemBreakpoint === null && (currentTabListWidth  + itemWidth ) >= tabListWidth ) {

							// Record at which point the item has been changed over
							tabItemBreakpoint = i;

						}
						


					});

					// console.log( fakerElement );


					// console.log('The Showing List: ');
					// console.log(showingList);
					// console.log('The Dropdown List: ');
					// console.log(dropdownList);

					// Destroy Faker Element before continuing.
					fakerElement.remove();

				} else {

					// Time to add all into the dropdown list!
					dropdownList = originalList;

				}




				// Return the sorted arrays back to the calling method so that it can be used
				// for re-rendering the front-end tabs that are to be displayed
				return {
					showingList: showingList,
					dropdownList: dropdownList
				};


			},
			createDropdownElement: function( el ) {

				var selector = el;

				// Now to find the different dropdown items that are going to 
				// be applied to the custom select option 
				var customDropdown = "";

					customDropdown += "</select>";




			},
			renderDropdownList: function( list ) {

				// Generate html list based on the list and return it to the calling method


				if( list.length > 0 ) {


					var html = "";

					$.each(list, function(i, o) {

						var listItem = $(this);

						if( listItem.is(settings.activeItem) ) {



							var hasActive = ' active';

						} else {
							var hasActive = '';
						}
												
						html += "<div class='__mst_tabs_dropdown_item" + hasActive + "' data-target='" + listItem.attr('data-target') + "'>";

							html += listItem.html();

						html += "</div>";

					});


					return html;


				} else {

					return "";

				}


			},
			onClickMore: function ( ev ) {

				var tabList = ev.data.tabList;
				var el = $( ev.currentTarget );
				var evTarget = $( ev.target );




				if( !evTarget.hasClass('__mst_tabs_dropdown_item')  && !el.hasClass('__mst_tabs_dropdown_item') ) {
				// console.log( ev );

					// Find the dropdown in the tablist
					var dropdownList = tabList.find('.__mst_tabs_dropdown');

					// Check to see if the element exists;
					if( dropdownList.length > 0 ) {

						// Firstly check to see if the curren button is already displayed;
						// if it is then disable it
						if(!dropdownList.hasClass('active')) {
							// Now to Display it via a class
							dropdownList.addClass('active');
						} else {

							dropdownList.removeClass('active');

						}


					} else {

						return;

					}

				} else {
					return;
				}




			},
			onClickMoreItem: function ( ev ) {


				// console.log('Now running the event!');


				var tabList = ev.data.tabList;
				var showingList = ev.data.showingList;
				var dropdownList = ev.data.dropdownList;

				var el = $( ev.target );

				// Go up several levels and then add an active class to the more button
				var moreButton = el.parents('li.__mst_tabs_more');


				if( showingList.length == 0 && dropdownList.length == originalList.length && $(window).outerWidth() <= settings.mobileBreakpoint ) {

					var dropdownTitle = moreButton.find('.__mst_tabs_dropdown_title');

					dropdownTitle.html( el.html() );

				}

				// Remove all active classes
				var allTabContent = $(tabList).siblings('.__mst_tab_content');

				allTabContent.removeClass('active');


				moreButton.addClass('active');

				var dropdownPar = el.parents('.__mst_tabs_dropdown');


				// Scroll through all of the active items and then disable them;
				// after disabling them then activate the current target
				var elTarget = $( el.attr('data-target') );


				var dropdownItems = el.siblings('.__mst_tabs_dropdown_item');

				$.each(dropdownItems, function(i, o) {


					var inItem = $(this);

					if(inItem.hasClass('active')) {

						inItem.removeClass('active');

						// Search for the target
						var inItemTarget = $( inItem.attr('data-target') );

						if( inItemTarget.length > 0 ) {

							inItemTarget.removeClass('active');

						}


					}

				});



				// Also run the same thing for the tab list 
				var tabListItems = tabList.find( settings.tabItemClass );

				$.each(tabListItems, function(i, o) {

					var inEl = $(this);

					if ( inEl.hasClass('active') ) {

						inEl.removeClass('active');

						var inElTarget = $( inEl.attr('data-target') );

						if ( inElTarget.length > 0 ) {

							inElTarget.removeClass('active');

						}

					}


				});



				// Now to display the current target element and classes.
				$.each(originalList, function(i, o) {

					if( $(this).attr('data-target') == el.attr('data-target') ) {
						// We have a match
						console.log('hit_dropdown');
						settings.activeItem = originalList[i];
					}

				});


				el.addClass('active');
				elTarget.addClass('active');


				// Before displaying the content to the user, remember to call a 
				// hide function on the dropdown
				pluginMethods.hideDropdownMenu( tabList );



			},
			onClickTab: function ( ev ) {

				// Pass over params from event Listener
				var tabList = ev.data.tabList;
				var showingList = ev.data.showingList;
				var dropdownList = ev.data.dropdownList;

				var el = $( ev.currentTarget );
				var tg = $( el.attr('data-target') );
				var sibs = el.siblings('.__mst_tab_list_item');
				var par = el.parents('.__mst_tabs');
				var content = par.find('.__mst_tab_content');



				// Remove all active classes
				var allTabContent = $(tabList).siblings('.__mst_tab_content');

				allTabContent.removeClass('active');

				// Also search for the more tab, then begin to remove the active state from it.
				var moreButton = el.siblings('li.__mst_tabs_more');

				moreButton.removeClass('active');


				$.each(originalList, function(i, o) {

					if( $(this).is( el)) {
						settings.activeItem = originalList[i];
						return;
					}

				});

				// Now run through all and remove the active class
				sibs.removeClass('active');
				el.addClass('active');
				content.removeClass('active');
				tg.addClass('active');


				// Also run through all of the list elements and find a more button
				var moreButton = tabList.find('li.__mst_tabs_more');

				if( moreButton.length > 0 ) {

					// A more button has been found.
					var moreItems = moreButton.find('.__mst_tabs_dropdown_item');

					$.each(moreItems, function(i , o) {

						var item = $(this);

						item.removeClass('active');


					});


				}




				// Also check to see if a more button exists, if so then ensure that the 
				// dropdown menu is hidden before continuing
				pluginMethods.hideDropdownMenu( tabList );


			},
			offClickDocument: function ( ev ) {

				var tabList = $( ev.data.tabList );

				var el = $( ev.currentTarget );
				var evTarget = $( ev.target );
				
				
				var match = null;

				var moreButton = tabList.find('li.__mst_tabs_more');


				var tabListSpawn = moreButton.find("*");
				tabListSpawn.push(moreButton[0]);


				$.each(tabListSpawn, function(i, o) {


					// Recursively check to see if the element is any of these
					var itemCheck = $(this);


					if( itemCheck.is(el) || itemCheck.is(evTarget) ) {

						// Houston we have a match with the target click
						match = i;
						return;

					}

				});

				if(match === null) {

					pluginMethods.hideDropdownMenu( tabList );

				}





			},
			hideDropdownMenu: function( tabList ) {

				// Debugging
				// console.log( ' Now calling hideDropdownMenu() fn...' );

				// Re-init jQuery tablist 
				tabList = $( tabList );

				var moreButton = tabList.find('li.__mst_tabs_more');


				if( moreButton.length > 0 ) {

					// console.log(' More Button has been found!');


					// The more button now exists; continue with the logic 
					var dropdownDom = moreButton.find('div.__mst_tabs_dropdown');


					if( dropdownDom.length > 0 ) {
						dropdownDom.removeClass('active');
						dropdownDom.removeClass('active');
						// console.log( dropdownDom );
					} else {

						// console.log('HOUSTON! We have a problem...');

					}




				} else {

					console.log('You have been disconnected from the server...');

				}


			},
			checkVisibility: function (element, options) {
		        if ($.contains(document, element[0])) {
		            var previousVisibility = options.previousVisibility;
		            var isVisible = element.is(':visible');
		            options.previousVisibility = isVisible;
		            var initialLoad = previousVisibility == null
		            if (initialLoad) {
		                if (options.runOnLoad) {
		                    options.callback(element, isVisible, initialLoad);
		                }
		            } else if (previousVisibility !== isVisible) {
		                options.callback(element, isVisible, initialLoad);
		            }

		            setTimeout(function() {
		                pluginMethods.checkVisibility(element, options);
		            }, options.frequency);
		        }
		    }


		};





		
		// Run the loop
		pluginMethods.__init( selectedObjects );


		// $(window).on('resize', function(ev) {
		// 	pluginMethods.__init( selectedObjects );
		// });





	};






}(jQuery));